/*
@file    FT800_config.c
@brief   Contains hardware abstraction functions for using the FT8xx
@version 2.5
@date    2016-10-06
@author  Rudolph Riedel

This file needs to be renamed to FT800_config.cpp for use with Arduino. 

@section History

2.2
- extracted from FT800_commands.c
- added ft800_pdn_set() and ft800_pdn_clear()
- replaced spi_flash_write() with fetch_flash_byte()

2.3
- added set of functions for Arduino

2.4
- switched from custom AVR8 to standard __AVR__ symbol to automatically generate plattform specific code

2.5
- added support for RH850
- switched to MISRA compliant comment-style

*/

#include "FT800_config.h"

#ifndef ARDUINO

	#if defined (__GNUC__)
	
		#if defined (__AVR__)

			void ft800_cs_set(void)
			{
				FT_CS_PORT &= ~FT_CS;	/* cs low */
			}

			void ft800_cs_clear(void)
			{
				FT_CS_PORT |= FT_CS;	/* cs high */
			}

			void ft800_pdn_set(void)
			{
				FT_PDN_PORT &= ~FT_PDN;	/* Power-Down low */
			}

			void ft800_pdn_clear(void)
			{
				FT_PDN_PORT |= FT_PDN;	/* Power-Down high */
			}

			void spi_transmit(uint8_t data)
			{
				SPDR = data; /* Start transmission */
				while(!(SPSR & (1<<SPIF))); /* Wait for transmission to complete - 1�s @ 8MHz SPI-Clock */
			}

			uint8_t spi_receive(uint8_t data)
			{
				SPDR = data; /* Start transmission */
				while(!(SPSR & (1<<SPIF))); /* Wait for transmission to complete - 1�s @ 8MHz SPI-CLock */
				return SPDR;
			}

			uint8_t fetch_flash_byte(const uint8_t *data)
			{
				return(pgm_read_byte_far(data));
			}
		
		#endif /* AVR */

		#if defined (__v851__)

			void ft800_pdn_set(void)
			{
				P0 &= ~(1u<<6);
			}

			void ft800_pdn_clear(void)
			{
				P0 |= (1u<<6);
			}

			void ft800_cs_set(void)
			{
				P8 &= ~(1u<<2); /* manually set chip-select to low */
			}

			void ft800_cs_clear(void)
			{
				P8 |= (1u<<2);  /* manually set chip-select to high */
			}

			void spi_transmit(uint8_t data)
			{
				CSIH0CTL0 = 0xC1; /* CSIH2PWR = 1;  CSIH2TXE=1; CSIH2RXE = 0; direct access mode  */
				CSIH0TX0H = data;	/* Start transmission */
				while(CSIH0STR0 & 0x00080);	/* Wait for transmission to complete - 800ns @ 10MHz SPI-Clock */
			}

			uint8_t spi_receive(uint8_t data)
			{
				CSIH0CTL0 = 0xE1; /* CSIH2PWR = 1;  CSIH2TXE=1; CSIH2RXE = 1; direct access mode  */
				CSIH0TX0H = data;	/* Start transmission */
				while(CSIH0STR0 & 0x00080);	/* Wait for transmission to complete - 800ns @ 10MHz SPI-Clock */
				return (uint8_t) CSIH0RX0H;
			}

			uint8_t fetch_flash_byte(const uint8_t *data)
			{
				return *data;
			}

		#endif /* RH850 */

	#endif /* GCC */
#endif /* !Arduino */


#ifdef ARDUINO

void ft800_cs_set(void)
{
	SPI.setDataMode(SPI_MODE0);
	digitalWrite(FT_CS, LOW);
}


void ft800_cs_clear(void)
{
	digitalWrite(FT_CS, HIGH);
}

void ft800_pdn_set(void)
{
	digitalWrite(FT_PDN, LOW);	/* Power-Down low */
}


void ft800_pdn_clear(void)
{
	digitalWrite(FT_PDN, HIGH);	/* Power-Down high */
}

void spi_transmit(uint8_t data)
{
	SPI.transfer(data);
}


uint8_t spi_receive(uint8_t data)
{
	return SPI.transfer(data);
}

uint8_t fetch_flash_byte(const uint8_t *data)
{
	return(pgm_read_byte_near(data));
}

#endif