/*
@file    tft.c
@brief   TFT handling functions for FT8xx_Test project, the layout is for 480x272 displays
@version 1.2
@date    2017-02-19
@author  Rudolph Riedel

@section History

1.0
- initial release

1.1
- added a simple .png image that is drawn beneath the clock just as example of how it could be done 

1.2
- rewrote parts of it for a big-picture display test, trimmed down to almost nothing

 */ 

#include "FT800.h"
#include "FT800_commands.h"

#include "tft_data.h"

/* memory-map defines */
#define MEM_LOGO 0x00000000 /* start-address of logo, needs 2242 bytes of memory */
#define MEM_PIC1 0x00000900 /* start of 100x100 pixel test image, needs 20000 bytes of memory */ 
#define MEM_DL_STATIC 0x000ff000 /* start-address of the static part of the display-list, upper 4k of gfx-mem */

uint32_t num_dl_static;

uint8_t tft_active = 0;

void initStaticBackground()
{
	ft800_cmd_dl(CMD_DLSTART); /* Start the display list */

	/* display the logo */
	ft800_cmd_dl(DL_BEGIN | FT_BITMAPS);
	ft800_cmd_setbitmap(MEM_LOGO, FT_L8, 38, 59);
	ft800_cmd_dl(VERTEX2F(420*16, 10*16));
	ft800_cmd_dl(DL_END);
	
	/* display the picture */
	ft800_cmd_dl(DL_BEGIN | FT_BITMAPS);
#if 0
	ft800_cmd_setbitmap(MEM_PIC1, FT_RGB565, 100, 100);
#endif
#if 1
	ft800_cmd_setbitmap(MEM_PIC1, FT_RGB565, 200, 200);
#endif
	ft800_cmd_dl(VERTEX2F(20*16, 40*16));
	ft800_cmd_dl(DL_END);

	ft800_cmd_dl(DL_COLOR_RGB | WHITE);
	ft800_cmd_text(10, 10, 27, 0, "FT81x Test");

	ft800_cmd_execute();
	while (ft800_busy());
	
	num_dl_static = ft800_memRead16(REG_CMD_DL);

	ft800_cmd_memcpy(MEM_DL_STATIC, FT_RAM_DL, num_dl_static);
	ft800_cmd_execute();
	while (ft800_busy());
}


void TFT_init(void)
{
	if(ft800_init() != 0)
	{
		tft_active = 1;

		/* send pre-recorded touch calibration values, HY50B */
		ft800_memWrite32(REG_TOUCH_TRANSFORM_A, 0xffff7f55);
		ft800_memWrite32(REG_TOUCH_TRANSFORM_B, 0x00000000);
		ft800_memWrite32(REG_TOUCH_TRANSFORM_C, 0x01f0d673);
		ft800_memWrite32(REG_TOUCH_TRANSFORM_D, 0x00000084);
		ft800_memWrite32(REG_TOUCH_TRANSFORM_E, 0x000054b9);
		ft800_memWrite32(REG_TOUCH_TRANSFORM_F, 0xffe0f006);

#if 0
 		/* calibrate touch and displays values to screen */
		ft800_cmd_dl(CMD_DLSTART);
		ft800_cmd_dl(DL_CLEAR_RGB | BLACK);
		ft800_cmd_dl(DL_CLEAR | CLR_COL | CLR_STN | CLR_TAG);
		ft800_cmd_text((FT_HSIZE/2), (FT_VSIZE/2), 27, FT_OPT_CENTER, "Please tap on the dot.");
		ft800_cmd_calibrate();
		ft800_cmd_dl(DL_DISPLAY);
		ft800_cmd_dl(CMD_SWAP);
		ft800_cmd_execute();

		while(ft800_busy() == 1);

		uint32_t touch_a, touch_b, touch_c, touch_d, touch_e, touch_f;

		touch_a = ft800_memRead32(REG_TOUCH_TRANSFORM_A);
		touch_b = ft800_memRead32(REG_TOUCH_TRANSFORM_B);
		touch_c = ft800_memRead32(REG_TOUCH_TRANSFORM_C);
		touch_d = ft800_memRead32(REG_TOUCH_TRANSFORM_D);
		touch_e = ft800_memRead32(REG_TOUCH_TRANSFORM_E);
		touch_f = ft800_memRead32(REG_TOUCH_TRANSFORM_F);

		ft800_cmd_dl(CMD_DLSTART);
		ft800_cmd_dl(DL_CLEAR_RGB | BLACK);
		ft800_cmd_dl(DL_CLEAR | CLR_COL | CLR_STN | CLR_TAG);
		ft800_cmd_dl(TAG(0));

		ft800_cmd_text(5, 30, 28, 0, "TOUCH_TRANSFORM_A:");
		ft800_cmd_text(5, 50, 28, 0, "TOUCH_TRANSFORM_B:");
		ft800_cmd_text(5, 70, 28, 0, "TOUCH_TRANSFORM_C:");
		ft800_cmd_text(5, 90, 28, 0, "TOUCH_TRANSFORM_D:");
		ft800_cmd_text(5, 110, 28, 0, "TOUCH_TRANSFORM_E:");
		ft800_cmd_text(5, 130, 28, 0, "TOUCH_TRANSFORM_F:");

#ifdef FT_81X_ENABLE
		ft800_cmd_setbase(16L); /* FT81x only */
#endif
		ft800_cmd_number(250, 30, 28, 0, touch_a);
		ft800_cmd_number(250, 50, 28, 0, touch_b);
		ft800_cmd_number(250, 70, 28, 0, touch_c);
		ft800_cmd_number(250, 90, 28, 0, touch_d);
		ft800_cmd_number(250, 110, 28, 0, touch_e);
		ft800_cmd_number(250, 130, 28, 0, touch_f);

		ft800_cmd_dl(DL_DISPLAY);	/* instruct the graphics processor to show the list */
		ft800_cmd_dl(CMD_SWAP); /* make this list active */
		ft800_cmd_execute();
#endif

		ft800_cmd_inflate(MEM_LOGO, logo, 406); /* load logo into gfx-memory and de-compress it */
		ft800_cmd_execute();
		while(ft800_busy() == 1);

#if 0
		ft800_cmd_loadimage(MEM_PIC1, FT_OPT_NODL, pngpic, 3867);
		ft800_cmd_execute();
		while(ft800_busy() == 1);
#endif

#if 1
		ft800_cmd_loadimage(MEM_PIC1, FT_OPT_NODL, testpic, 17038);
//		ft800_cmd_execute();
//		while(ft800_busy() == 1);
#endif

		initStaticBackground();
	}
}


/*
	dynamic portion of display-handling, meant to be called every 10ms
	divided into two sections:
		- handling of touch-events and variables
		- sending a new display-list to the FT8xx
*/
void TFT_loop(void)
{
    static uint8_t delay;

	if(tft_active != 0)
	{
	  	switch(delay)
		{
			case 0:
				break;

			case 1:
				ft800_cmd_dl(CMD_DLSTART); /* start the display list */
				ft800_cmd_dl(DL_CLEAR_RGB | BLACK); /* set the default clear color to white */
				ft800_cmd_dl(DL_CLEAR | CLR_COL | CLR_STN | CLR_TAG); /* clear the screen - this and the previous prevent artifacts between lists, Attributes are the color, stencil and tag buffers */
				ft800_cmd_dl(TAG(0));

				ft800_cmd_append(MEM_DL_STATIC, num_dl_static); /* insert static part of display-list from copy in gfx-mem */

				ft800_cmd_dl(DL_DISPLAY);	/* instruct the graphics processor to show the list */
				ft800_cmd_dl(CMD_SWAP); /* make this list active */

				ft800_cmd_execute();
				break;
		}

		delay++;
		delay &= 0x01; /* -> delay toggles between 0 and 1 */
	}
}
